@@ -117,5 +117,5 @@ obj.parse = node('object',
 _json = node('json',
              ('value', value)) # alt(obj, array)),
 
-json = seq2L(seq2R(whitespace, _json),
+json = seq2L(seq2R(whitespace, cut('json value', _json)),
              cut('unparsed input remaining', not0(item)))
